import {
  Button,
  Card,
  Checkbox,
  Form,
  Select,
  Skeleton,
  Space,
  Switch,
  Typography,
  notification,
} from "antd";
import { myFetch } from "../../utils";
import { useTranslation } from "react-i18next";
import { useEffect, useRef, useState } from "react";
import {
  RequestState,
  RequestStateItem,
} from "../../Components/MyRequestStateItem";
import { LogoutOutlined } from "@ant-design/icons";

export default function UserProfile({ setUserSession }) {
  const { t, i18n } = useTranslation();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const [form] = Form.useForm();

  const [requestState, setRequestState] = useState(RequestState.INIT);
  const [isRequestingLogout, setIsRequestingLogout] = useState(false);
  const delayTimeout = useRef();

  const language = Form.useWatch("language", form);
  const analytics = Form.useWatch("analytics", form);

  useEffect(() => {
    myFetch({
      url: "/user/profile/settings",
      method: "GET",
      notificationApi: notificationApi,
      t: t,
    })
      .then((data) => {
        form.setFieldsValue({
          language: data.language,
          analytics: data.analytics_enabled,
        });
      })
      .catch((errStatus) => {
        console.log(errStatus);
      });
  }, []);

  useEffect(() => {
    if (language === undefined || analytics === undefined) return;

    if (RequestState.INIT === requestState) {
      setRequestState(RequestState.NOTHING);
      return;
    }

    setRequestState(RequestState.REQUESTING);

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      myFetch({
        url: "/user/profile/settings",
        method: "POST",
        body: {
          language: language,
          analyticsEnabled: analytics,
        },
        notificationApi: notificationApi,
        t: t,
      })
        .then(() => setRequestState(RequestState.SUCCESS))
        .catch((errStatus) => {
          console.log(errStatus);
          setRequestState(RequestState.FAILED);
        });
    }, 500);
  }, [language, analytics]);

  return (
    <>
      {notificationContextHolder}

      <Card
        title={t("userProfile.title")}
        extra={
          <Space>
            <RequestStateItem
              state={requestState}
              setRequestState={setRequestState}
            />

            <Button
              type="primary"
              loading={isRequestingLogout}
              icon={<LogoutOutlined />}
              onClick={() => {
                setIsRequestingLogout(true);

                myFetch({
                  url: "/user/auth/logout",
                  method: "DELETE",
                  notificationApi: notificationApi,
                  t: t,
                })
                  .then(() => {
                    setUserSession();
                    window.location.href = "/";
                  })
                  .catch(() => setIsRequestingLogout(false));
              }}
            >
              {t("common.button.logout")}
            </Button>
          </Space>
        }
      >
        <Form layout="horizontal" form={form}>
          <Form.Item name="language" label={t("userProfile.language")}>
            {requestState === RequestState.INIT ? (
              <Skeleton.Input active block></Skeleton.Input>
            ) : (
              <Select
                style={{ width: "100%" }}
                options={[
                  {
                    value: "en",
                    label: "English",
                  },
                  {
                    value: "de",
                    label: "Deutsch",
                  },
                ]}
                onChange={(e) => i18n.changeLanguage(e)}
              />
            )}
          </Form.Item>

          <Form.Item
            name="analytics"
            valuePropName="checked"
            label={t("userProfile.analytics")}
            extra={t("userProfile.analyticsDescription")}
          >
            {requestState === RequestState.INIT ? (
              <Skeleton.Button shape="round" active />
            ) : (
              <Switch>{t("userProfile.analytics")}</Switch>
            )}
          </Form.Item>
        </Form>
      </Card>
    </>
  );
}
